import json
import logging
import os
import pathlib
import shutil
import sys

from nearuplib.constants import NODE_PID_FILE, LOCALNET_FOLDER, LOCALNET_LOGS_FOLDER
from nearuplib.nodelib import run_binary, proc_name_from_pid, is_neard_running
from nearuplib import util


def run(binary_path,
        home,
        num_nodes,
        num_shards,
        override,
        verbose=True,
        interactive=False):
    home = pathlib.Path(home)

    if home.exists():
        if util.prompt_bool_flag(
                'Would you like to remove data from the previous localnet run?',
                override,
                interactive=interactive):
            logging.info("Removing old data.")
            shutil.rmtree(home)
    elif interactive:
        print(
            util.wraptext('''
            Starting localnet NEAR nodes.  This is a test network entirely local
            to this machine.  Validators and non-validating nodes will be
            started, and will communicate with each other on localhost,
            producing blocks on top of a genesis block generated locally.
        '''))
        print()

    if not home.exists():
        num_nodes = util.prompt_flag(
            'How many validator nodes would you like to initialize this localnet with?',
            num_nodes,
            default=4,
            interactive=interactive,
            type=int,
        )
        num_shards = util.prompt_flag(
            'How many shards would you like to initialize this localnet with?'
            '\nSee https://near.org/papers/nightshade/#sharding-basics',
            num_shards,
            default=1,
            interactive=interactive,
            type=int,
        )
        fixed_shards = False
        if num_shards > 1:
            fixed_shards = util.prompt_bool_flag(
                'Would you like to setup fixed accounts per each shard (shard0, shard1)?',
                False,
                interactive=interactive,
            )
        archival_nodes = util.prompt_bool_flag(
            "Should these nodes be archival nodes (keep full history)?",
            False,
            interactive=interactive)

        run_binary(binary_path,
                   home,
                   'localnet',
                   shards=num_shards,
                   validators=num_nodes,
                   fixed_shards=fixed_shards,
                   archival_nodes=archival_nodes,
                   print_command=interactive).wait()

    # Edit args files
    num_nodes = 0
    while True:
        path = home / f'node{num_nodes}' / 'config.json'
        if not path.exists():
            break

        data = json.loads(path.read_text())
        data['rpc']['addr'] = f'0.0.0.0:{3030 + num_nodes}'
        data['network']['addr'] = f'0.0.0.0:{24567 + num_nodes}'
        path.write_text(json.dumps(data, indent=2))
        num_nodes += 1

    # Load public key from first node
    data = json.loads((home / 'node0' / 'node_key.json').read_text())
    public_key = data['public_key']

    # Recreate log folder
    shutil.rmtree(LOCALNET_LOGS_FOLDER, ignore_errors=True)
    os.mkdir(LOCALNET_LOGS_FOLDER)

    # Spawn network
    with open(NODE_PID_FILE, 'w') as pid_fd:
        for i in range(0, num_nodes):
            proc = run_binary(
                binary_path,
                os.path.join(home, f'node{i}'),
                'run',
                verbose=verbose,
                boot_nodes=f'{public_key}@127.0.0.1:24567' if i > 0 else None,
                output=os.path.join(LOCALNET_LOGS_FOLDER, f'node{i}'),
                print_command=interactive)
            proc_name = proc_name_from_pid(proc.pid)
            pid_fd.write(f'{proc.pid}|{proc_name}|localnet\n')

    logging.info('Localnet was spawned successfully...')
    logging.info(f'Localnet logs written in: {LOCALNET_LOGS_FOLDER}')
    logging.info('Check localnet status at http://127.0.0.1:3030/status')


def entry(binary_path, home, num_nodes, num_shards, override, verbose,
          interactive):
    if binary_path:
        binary_path = os.path.join(binary_path, 'neard')
    else:
        uname = os.uname()[0]
        binary_path = os.path.join(LOCALNET_FOLDER, 'neard')
        if not os.path.exists(LOCALNET_FOLDER):
            os.makedirs(LOCALNET_FOLDER)
        util.download_binaries('localnet', uname)

    if is_neard_running():
        sys.exit(1)

    run(binary_path, home, num_nodes, num_shards, override, verbose,
        interactive)



import json
import os
import shutil
from pathlib import Path

import pytest

from nearuplib.nodelib import check_and_setup

ACCOUNT_ID = 'mock.nearup.account'
BETANET_HOME = os.path.expanduser('~/.near/betanet')
BETANET_BINARY_PATH = os.path.expanduser('~/.near/near/betanet')
LOCALNET_HOME = os.path.expanduser('~/.near/localnet')
LOCALNET_BINARY_PATH = os.path.expanduser('~/.near/near/localnet')


def cleanup():
    if os.path.exists(LOCALNET_HOME):
        shutil.rmtree(LOCALNET_HOME)

    if os.path.exists(BETANET_HOME):
        shutil.rmtree(BETANET_HOME)


def setup_module(module):  # pylint: disable=W0613
    cleanup()
    os.makedirs(BETANET_HOME)
    os.makedirs(LOCALNET_HOME)


def teardown_module(module):  # pylint: disable=W0613
    cleanup()


def write_config_files(home):
    Path(f'{home}/node_key.json').touch()
    Path(f'{home}/config.json').touch()
    Path(f'{home}/genesis.json').touch()


def write_genesis(home, chain_id):
    with open(f'{home}/genesis.json', 'w') as genesis:
        json.dump({'chain_id': chain_id}, genesis)


def test_check_and_setup_localnet_existing_config():
    """Home directory exists with config"""
    write_config_files(LOCALNET_HOME)
    write_genesis(LOCALNET_HOME, 'localnet')

    try:
        check_and_setup(LOCALNET_BINARY_PATH, LOCALNET_HOME, 'localnet', None)
    except Exception as ex:
        pytest.fail(f'unexpected expection {ex}')


def test_check_and_setup_betanet_existing_config():
    """Home directory for betanet exists with config"""
    write_config_files(BETANET_HOME)
    write_genesis(BETANET_HOME, 'betanet')

    try:
        check_and_setup(BETANET_BINARY_PATH, BETANET_HOME, 'betanet',
                        ACCOUNT_ID)
    except Exception as ex:
        pytest.fail(f'unexpected exception {ex}')



import json
import os
import shutil

from nearuplib.nodelib import download_binaries, init_near

HOME = os.path.expanduser('~/.near/betanet')
BINARY_PATH = os.path.expanduser('~/.nearup/near/betanet')
NEARUP_PATH = os.path.expanduser('~/.nearup/')
ACCOUNT_ID = 'mock.nearup.account'


def setup_module(module):  # pylint: disable=W0613
    if os.path.exists(HOME):
        shutil.rmtree(HOME)

    if os.path.exists(NEARUP_PATH):
        shutil.rmtree(NEARUP_PATH)

    os.makedirs(BINARY_PATH)
    os.makedirs(HOME)

    uname = os.uname()[0]
    download_binaries('betanet', uname)


def assert_node_key():
    node_key_path = os.path.join(HOME, 'node_key.json')
    assert os.path.exists(node_key_path)

    with open(node_key_path) as node_key:
        data = json.loads(node_key.read())
        assert data['account_id'] == 'node'
        assert 'public_key' in data
        assert 'secret_key' in data


def assert_validator_key():
    validator_key_path = os.path.join(HOME, 'validator_key.json')
    assert os.path.exists(validator_key_path)

    with open(validator_key_path) as node_key:
        data = json.loads(node_key.read())
        assert data['account_id'] == ACCOUNT_ID
        assert 'public_key' in data
        assert 'secret_key' in data


def test_init_near():
    init_near(HOME, BINARY_PATH, 'betanet', ACCOUNT_ID)
    assert_node_key()
    assert_validator_key()



import os
import shutil

import pytest
import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from nearuplib.nodelib import restart_nearup, setup_and_run, stop_nearup
from nearuplib.constants import LOGS_FOLDER

NEAR_DIR = os.path.expanduser('~/.near/betanet')
NEARUP_DIR = '~/.nearup'

NEARUP_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    '../nearup',
)


def cleanup():
    if os.path.exists(NEAR_DIR):
        shutil.rmtree(NEAR_DIR)

    if os.path.exists(NEARUP_DIR):
        shutil.rmtree(NEARUP_DIR)

    if os.path.exists(LOGS_FOLDER):
        shutil.rmtree(LOGS_FOLDER)


def setup_module(module):  # pylint: disable=W0613
    cleanup()

    if not os.path.exists(LOGS_FOLDER):
        os.makedirs(LOGS_FOLDER)


def teardown_module(module):  # pylint: disable=W0613
    cleanup()


def test_nearup_still_runnable():
    setup_and_run(binary_path='',
                  home_dir=NEAR_DIR,
                  chain_id='betanet',
                  boot_nodes='',
                  verbose=True,
                  watcher=False)

    retry_strategy = Retry(total=5, backoff_factor=5)
    http = requests.Session()
    http.mount("http://", HTTPAdapter(max_retries=retry_strategy))

    resp = http.get('http://localhost:3030/status')
    assert resp.status_code == 200
    assert resp.text

    stop_nearup(keep_watcher=True)

    with pytest.raises(requests.exceptions.ConnectionError):
        requests.get('http://localhost:3030/status')

    restart_nearup('betanet', NEARUP_PATH, NEAR_DIR, keep_watcher=True)

    resp = http.get('http://localhost:3030/status')
    assert resp.status_code == 200
    assert resp.text

    stop_nearup(keep_watcher=True)

    with pytest.raises(requests.exceptions.ConnectionError):
        requests.get('http://localhost:3030/status')



import functools


class NetworkError(Exception):
    """Designates a non-fatal networking error"""


def capture_as(exception):
    """Capture any exception raised by the wrapped function and re-raise as the given exception."""

    def wrap(func):

        @functools.wraps(func)
        def wrapped(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as ex:
                raise exception() from ex

        return wrapped

    return wrap



from nearuplib.tailer import next_logname


def test_next_logname():
    assert next_logname(
        'test-data/incrementlog.log') == 'test-data/incrementlog.log.2'
    assert next_logname('test-data/justlog.log') == 'test-data/justlog.log.1'
    assert next_logname(
        'test-data/nonexistinglog.log') == 'test-data/nonexistinglog.log'



import logging
import os
import site
import sys

from subprocess import Popen

import psutil

from nearuplib.constants import DEFAULT_WAIT_TIMEOUT, WATCHER_PID_FILE


def check_watcher_file():
    if not os.path.exists(WATCHER_PID_FILE):
        return False

    with open(WATCHER_PID_FILE) as pid_file:
        try:
            pid = int(pid_file.readline().strip())
        except Exception:
            logging.error(
                f"Nearup watcher PID file {WATCHER_PID_FILE} has unexpected content."
            )
            return True

        logging.warning(
            f"Old Nearup watcher PID file {WATCHER_PID_FILE} found.")

        try:
            os.kill(pid, 0)
        except OSError:
            return False
        else:
            logging.error("Nearup watcher is running.")
            return True


def is_watcher_running():
    if check_watcher_file():
        logging.error("Run nearup stop or kill the process manually!")
        logging.warning(f"If this is a mistake, remove {WATCHER_PID_FILE}")
        return True
    return False


def run_watcher(net, path=os.path.join(site.USER_BASE, 'bin/watcher'), home=''):
    logging.info("Starting the nearup watcher...")

    if is_watcher_running():
        sys.exit(1)

    if not os.path.exists(path):
        logging.error(
            "Delete current nearup and install the new with `pip3 install --user nearup`"
        )
        logging.error(
            "To run nearup locally use: `pip3 install --user .` from root directory"
        )
        sys.exit(1)

    proc = Popen(['python3', path, 'run', net, home])

    with open(WATCHER_PID_FILE, 'w') as watcher_pid_file:
        watcher_pid_file.write(str(proc.pid))


def stop_watcher(timeout=DEFAULT_WAIT_TIMEOUT):
    try:
        if os.path.exists(WATCHER_PID_FILE):
            with open(WATCHER_PID_FILE) as pid_file:
                pid = int(pid_file.read())
                process = psutil.Process(pid)
                logging.info(
                    f'Stopping near watcher {process.name()} with pid {pid}...')

                try:
                    process.terminate()
                    process.wait(timeout=timeout)
                except psutil.TimeoutExpired:
                    logging.warning(
                        'Watcher taking a long time to terminate...')
                    logging.warning('Killing watcher with pid {pid}')
                    process.kill()

                os.remove(WATCHER_PID_FILE)
        else:
            logging.info("Nearup watcher is not running...")
    except Exception as ex:
        logging.error(f"There was an error while stopping watcher: {ex}")



import os

LOCALNET_FOLDER = os.path.expanduser("~/.nearup/near/localnet")
LOGS_FOLDER = os.path.expanduser('~/.nearup/logs')
LOCALNET_LOGS_FOLDER = os.path.expanduser("~/.nearup/logs/localnet")
NODE_PID_FILE = os.path.expanduser('~/.nearup/node.pid')
WATCHER_PID_FILE = os.path.expanduser('~/.nearup/watcher.pid')
DEFAULT_WAIT_TIMEOUT = 30

S3_BUCKETS = {
    'default': 'build.nearprotocol.com',
    'mainnet': 'build.nearprotocol.com',
    'testnet': 'build.nearprotocol.com',
    'betanet': 'build.nearprotocol.com',
    'guildnet': 'build.openshards.io',
    'shardnet': 'build.nearprotocol.com',
}
